from qpd._parser.sql import QPDSql
from qpd._parser.utils import VisitorContext
from qpd._parser.visitors import StatementVisitor
from qpd.qpd_engine import QPDEngine
from qpd.workflow import QPDWorkflow, QPDWorkflowContext
from typing import Any


def run_sql(engine: QPDEngine, sql: str, **dfs: Any) -> Any:
    qsql = QPDSql(sql, "singleStatement")
    ctx = QPDWorkflowContext(engine, dfs)
    wf = QPDWorkflow(ctx)
    v = StatementVisitor(VisitorContext(sql=qsql, workflow=wf, dfs=wf.dfs,))
    wf.assemble_output(v.visit(qsql.tree))
    wf.run()
    return ctx.result



from typing import Any

import pandas as pd

from qpd.dataframe import Column, DataFrame
from qpd_pandas import QPDPandasEngine
from qpd_test.sql_suite import SQLTests


class SQLTests(SQLTests.Tests):
    def to_pandas_df(self, df: Any) -> pd.DataFrame:
        if isinstance(df, DataFrame):
            return self.op.to_native(df)
        elif isinstance(df, pd.DataFrame):
            return df
        else:
            raise ValueError(df)

    def to_pandas_series(self, col: Column) -> pd.Series:
        return col.native

    def make_qpd_engine(self):
        return QPDPandasEngine()

    def to_native_df(self, data: Any, columns: Any) -> Any:  # pragma: no cover
        if isinstance(data, pd.DataFrame):
            return data
        return pd.DataFrame(data, columns=columns)



from setuptools import setup, find_packages
from qpd_version import __version__


with open("README.md") as f:
    LONG_DESCRIPTION = f.read()

setup(
    name="qpd",
    version=__version__,
    packages=find_packages(),
    description="Query Pandas Using SQL",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    license="Apache-2.0",
    author="Han Wang",
    author_email="goodwanghan@gmail.com",
    keywords="pandas sql",
    url="http://github.com/goodwanghan/qpd",
    install_requires=[
        "pandas>=1.0.2",
        "triad",
        "adagio",
        "sqlalchemy",
        "antlr4-python3-runtime",
    ],
    extras_require={
        "dask": ["dask[dataframe]", "cloudpickle>=1.4.0"],
        "all": ["dask[dataframe]", "cloudpickle>=1.4.0"],
    },
    classifiers=[
        # "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
    ],
    python_requires=">=3.6",
)
